@@ -1,15 +1,15 @@
 import io
 import json
+import uuid
 from typing import IO, Any, DefaultDict, Dict, List, Optional, Tuple, Union
 
 import requests
-from aqueduct.constants.enums import ExecutionStatus, ServiceType
+from aqueduct.constants.enums import ExecutionStatus, RuntimeType, ServiceType
 from aqueduct.error import (
     AqueductError,
     ClientValidationError,
     InternalServerError,
     InvalidRequestError,
-    InvalidUserArgumentException,
     NoConnectedIntegrationsException,
     ResourceNotFoundError,
     UnprocessableEntityError,
@@ -17,9 +17,9 @@ from aqueduct.error import (
 from aqueduct.logger import logger
 from aqueduct.models.dag import DAG
 from aqueduct.models.integration import Integration, IntegrationInfo
-from aqueduct.models.operators import LoadSpec, ParamSpec, RelationalDBLoadParams, S3LoadParams
+from aqueduct.models.operators import ParamSpec
 from aqueduct.utils.serialization import deserialize
-from pkg_resources import parse_version, require
+from pkg_resources import get_distribution, parse_version
 
 from ..integrations.connect_config import IntegrationConfig
 from .response_helpers import (
@@ -29,6 +29,7 @@ from .response_helpers import (
 )
 from .response_models import (
     DeleteWorkflowResponse,
+    DynamicEngineStatusResponse,
     GetVersionResponse,
     GetWorkflowResponse,
     ListWorkflowResponseEntry,
@@ -50,6 +51,7 @@ class APIClient:
 
     GET_VERSION_ROUTE = "/api/version"
     CONNECT_INTEGRATION_ROUTE = "/api/integration/connect"
+    DELETE_INTEGRATION_ROUTE_TEMPLATE = "/api/integration/%s/delete"
     PREVIEW_ROUTE = "/api/preview"
     REGISTER_WORKFLOW_ROUTE = "/api/workflow/register"
     REGISTER_AIRFLOW_WORKFLOW_ROUTE = "/api/workflow/register/airflow"
@@ -67,6 +69,9 @@ class APIClient:
     NODE_POSITION_ROUTE = "/api/positioning"
     EXPORT_FUNCTION_ROUTE = "/api/function/%s/export"
 
+    GET_DYNAMIC_ENGINE_STATUS_ROUTE = "/api/integration/dynamic-engine/status"
+    EDIT_DYNAMIC_ENGINE_ROUTE_TEMPLATE = "/api/integration/dynamic-engine/%s/edit"
+
     # Auth header
     API_KEY_HEADER = "api-key"
 
@@ -156,7 +161,7 @@ class APIClient:
 
     def _validate_server_version(self, server_version: str) -> None:
         """Checks that the SDK and the server versions match."""
-        sdk_version = require("aqueduct-sdk")[0].version
+        sdk_version = get_distribution("aqueduct-sdk").version
         if parse_version(server_version) > parse_version(sdk_version):
             raise ClientValidationError(
                 "The SDK is outdated, it is using version %s, while the server is of version %s. "
@@ -270,11 +275,103 @@ class APIClient:
         resp = requests.post(url, url, headers=headers)
         self.raise_errors(resp)
 
+    def get_dynamic_engine_status_by_dag(
+        self,
+        dag: DAG,
+    ) -> Dict[str, DynamicEngineStatusResponse]:
+        """Makes a request against the /api/integration/dynamic-engine/status endpoint.
+           If an integration id does not correspond to a dynamic integration, the response won't
+           have an entry for that integration.
+
+        Args:
+            dag:
+                The DAG object. We will extract the engine integration IDs and send them
+                to the backend to retrieve their status. Currently, we are only interested in
+                the status of dynamic engines.
+
+        Returns:
+            A DynamicEngineStatusResponse object, parsed from the backend endpoint's response.
+        """
+        engine_integration_ids = set()
+
+        dag_engine_config = dag.engine_config
+        if dag_engine_config.type == RuntimeType.K8S:
+            assert dag_engine_config.k8s_config is not None
+            engine_integration_ids.add(str(dag_engine_config.k8s_config.integration_id))
+        for op in dag.operators.values():
+            if op.spec.engine_config and op.spec.engine_config.type == RuntimeType.K8S:
+                assert op.spec.engine_config.k8s_config is not None
+                engine_integration_ids.add(str(op.spec.engine_config.k8s_config.integration_id))
+
+        return self.get_dynamic_engine_status(list(engine_integration_ids))
+
+    def get_dynamic_engine_status(
+        self,
+        engine_integration_ids: List[str],
+    ) -> Dict[str, DynamicEngineStatusResponse]:
+        """Makes a request against the /api/integration/dynamic-engine/status endpoint.
+           If an integration id does not correspond to a dynamic integration, the response won't
+           have an entry for that integration.
+
+        Args:
+            engine_integration_ids:
+                A list of engine integration IDs. Currently, we are only interested in
+                the status of dynamic engines.
+
+        Returns:
+            A DynamicEngineStatusResponse object, parsed from the backend endpoint's response.
+        """
+        headers = self._generate_auth_headers()
+        headers["integration-ids"] = json.dumps(engine_integration_ids)
+
+        url = self.construct_full_url(self.GET_DYNAMIC_ENGINE_STATUS_ROUTE)
+        resp = requests.get(url, headers=headers)
+        self.raise_errors(resp)
+
+        return {
+            dynamic_engine_status["name"]: DynamicEngineStatusResponse(**dynamic_engine_status)
+            for dynamic_engine_status in resp.json()
+        }
+
+    def edit_engine(
+        self,
+        action: str,
+        integration_id: str,
+    ) -> None:
+        """Makes a request against the /api/integration/dynamic-engine/{integrationId}/edit endpoint.
+
+        Args:
+            integration_id:
+                The engine integration ID.
+        """
+        headers = self._generate_auth_headers()
+        headers["action"] = action
+
+        url = self.construct_full_url(self.EDIT_DYNAMIC_ENGINE_ROUTE_TEMPLATE % integration_id)
+
+        if action == "create" or action == "delete":
+            resp = requests.post(url, headers=headers)
+        else:
+            raise InvalidRequestError(
+                "Invalid action %s for interacting with dynamic engine." % action
+            )
+
+        self.raise_errors(resp)
+
+    def delete_integration(
+        self,
+        integration_id: uuid.UUID,
+    ) -> None:
+        url = self.construct_full_url(self.DELETE_INTEGRATION_ROUTE_TEMPLATE % integration_id)
+        headers = self._generate_auth_headers()
+        resp = requests.post(url, headers=headers)
+        self.raise_errors(resp)
+
     def preview(
         self,
         dag: DAG,
     ) -> PreviewResponse:
-        """Makes a request against the /preview endpoint.
+        """Makes a request against the /api/preview endpoint.
 
         Args:
             dag:
