@@ -28,6 +28,7 @@ from aqueduct.error import (
 from aqueduct.flow import Flow
 from aqueduct.github import Github
 from aqueduct.integrations.airflow_integration import AirflowIntegration
+from aqueduct.integrations.aws_integration import AWSIntegration
 from aqueduct.integrations.connect_config import (
     BaseConnectionConfig,
     IntegrationConfig,
@@ -41,6 +42,7 @@ from aqueduct.integrations.lambda_integration import LambdaIntegration
 from aqueduct.integrations.mongodb_integration import MongoDBIntegration
 from aqueduct.integrations.s3_integration import S3Integration
 from aqueduct.integrations.salesforce_integration import SalesforceIntegration
+from aqueduct.integrations.spark_integration import SparkIntegration
 from aqueduct.integrations.sql_integration import RelationalDBIntegration
 from aqueduct.logger import logger
 from aqueduct.models.config import FlowConfig
@@ -282,6 +284,11 @@ class Client:
                 Either a dictionary or an IntegrationConnectConfig object that contains the
                 configuration credentials needed to connect.
         """
+        if service == ServiceType.AWS:
+            raise InvalidUserArgumentException(
+                "Support for service type AWS is not ready yet. Please stay tuned!"
+            )
+
         if service not in ServiceType:
             raise InvalidUserArgumentException(
                 "Service argument must match exactly one of the enum values in ServiceType (case-sensitive)."
@@ -308,6 +315,25 @@ class Client:
         globals.__GLOBAL_API_CLIENT__.connect_integration(name, service, config)
         logger().info("Successfully connected to new %s integration `%s`." % (service, name))
 
+    def delete_integration(
+        self,
+        name: str,
+    ) -> None:
+        """Deletes the integration from Aqueduct.
+
+        Args:
+            name:
+                The name of the integration to delete.
+        """
+        existing_integrations = globals.__GLOBAL_API_CLIENT__.list_integrations()
+        if name not in existing_integrations.keys():
+            raise InvalidIntegrationException("Not connected to integration %s!" % name)
+
+        globals.__GLOBAL_API_CLIENT__.delete_integration(existing_integrations[name].id)
+
+        # Update the connected integrations cached on this object.
+        self._connected_integrations = globals.__GLOBAL_API_CLIENT__.list_integrations()
+
     def list_integrations(self) -> Dict[str, IntegrationInfo]:
         """Retrieves a dictionary of integrations the client can use.
 
@@ -329,6 +355,8 @@ class Client:
         LambdaIntegration,
         MongoDBIntegration,
         DatabricksIntegration,
+        SparkIntegration,
+        AWSIntegration,
     ]:
         """Retrieves a connected integration object.
 
@@ -392,6 +420,19 @@ class Client:
             return DatabricksIntegration(
                 metadata=integration_info,
             )
+        elif integration_info.service == ServiceType.SPARK:
+            return SparkIntegration(
+                metadata=integration_info,
+            )
+        elif integration_info.service == ServiceType.AWS:
+            dynamic_k8s_integration_name = "%s:k8s" % name
+            dynamic_k8s_integration_info = self._connected_integrations[
+                dynamic_k8s_integration_name
+            ]
+            return AWSIntegration(
+                metadata=integration_info,
+                k8s_integration_metadata=dynamic_k8s_integration_info,
+            )
         else:
             raise InvalidIntegrationException(
                 "This method does not support loading integration of type %s"
