@@ -78,6 +78,8 @@ class ServiceType(str, Enum, metaclass=MetaEnum):
     DATABRICKS = "Databricks"
     EMAIL = "Email"
     SLACK = "Slack"
+    SPARK = "Spark"
+    AWS = "AWS"
 
 
 class RelationalDBServices(str, Enum, metaclass=MetaEnum):
@@ -190,15 +192,29 @@ class ExecutionMode(str, Enum, metaclass=MetaEnum):
     LAZY = "lazy"
 
 
-class RuntimeType(Enum, metaclass=MetaEnum):
+class RuntimeType(str, Enum, metaclass=MetaEnum):
     AQUEDUCT = "aqueduct"
+    AQUEDUCT_CONDA = "aqueduct_conda"
     AIRFLOW = "airflow"
     K8S = "k8s"
     LAMBDA = "lambda"
     DATABRICKS = "databricks"
+    SPARK = "spark"
 
 
 class NotificationLevel(Enum, metaclass=MetaEnum):
     SUCCESS = "success"
     WARNING = "warning"
     ERROR = "error"
+
+
+class SparkRuntimeType(str, Enum, metaclass=MetaEnum):
+    DATABRICKS = "databricks"
+    SPARK = "spark"
+
+
+class K8sClusterStatusType(str, Enum, metaclass=MetaEnum):
+    CREATING = "Creating"
+    ACTIVE = "Active"
+    TERMINATING = "Terminating"
+    TERMINATED = "Terminated"
