@@ -111,14 +111,32 @@ def wrap_spec(
     operator_id = generate_uuid()
     output_artifact_ids = [generate_uuid() for _ in output_artifact_type_hints]
 
+    overwrite_existing_op = True
+    colliding_op = dag.get_operator(with_name=op_name)
+    if colliding_op is not None:
+        # There is already an operator with the same name.
+        # If the input artifacts are different for this new operator
+        # and the existing operator, then a new operator should be added.
+        # Otherwise, the existing operator should be overwritten.
+        input_artifact_ids = [input_artf.id() for input_artf in input_artifacts]
+        if input_artifact_ids != colliding_op.inputs:
+            overwrite_existing_op = False
+
+    original_op_name = op_name
     op_name, artifact_names = resolve_op_and_artifact_names(
         dag,
         op_name,
-        overwrite_existing_op_name=True,
+        overwrite_existing_op_name=overwrite_existing_op,
         candidate_artifact_names=output_artifact_names,
         num_outputs=len(output_artifact_ids),
     )
 
+    if colliding_op is not None and op_name != original_op_name:
+        print(
+            "Warning: There is already an operator named %s so this operator is being renamed to %s"
+            % (original_op_name, op_name)
+        )
+
     apply_deltas_to_dag(
         dag,
         deltas=[
@@ -350,6 +368,66 @@ def _convert_memory_string_to_mbs(memory_str: str) -> int:
     return multiplier * int(memory_scalar_str)
 
 
+def _update_operator_spec_with_engine(
+    spec: OperatorSpec,
+    engine: Optional[str] = None,
+) -> None:
+    if engine is not None:
+        if globals.__GLOBAL_API_CLIENT__ is None:
+            raise InvalidUserActionException(
+                "Aqueduct Client was not instantiated! Please create a client and retry."
+            )
+
+        spec.engine_config = generate_engine_config(
+            globals.__GLOBAL_API_CLIENT__.list_integrations(),
+            engine,
+        )
+
+
+def _update_operator_spec_with_resources(
+    spec: OperatorSpec,
+    resources: Optional[Dict[str, Any]] = None,
+) -> None:
+    if resources is not None:
+        if not isinstance(resources, Dict) or any(not isinstance(k, str) for k in resources):
+            raise InvalidUserArgumentException("`resources` must be a dictionary with string keys.")
+
+        num_cpus = resources.get(NUM_CPUS_KEY)
+        memory = resources.get(MEMORY_KEY)
+        gpu_resource_name = resources.get(GPU_RESOURCE_NAME_KEY)
+
+        if num_cpus is not None and (not isinstance(num_cpus, int) or num_cpus < 0):
+            raise InvalidUserArgumentException(
+                "`num_cpus` value must be set to a positive integer."
+            )
+
+        # `memory` value can be either an int (in MBs) or a string. We will convert it into an integer
+        # representing the number of MBs.
+        if memory is not None:
+            if not isinstance(memory, int) and not isinstance(memory, str):
+                raise InvalidUserArgumentException(
+                    "`memory` value must be either an integer or string."
+                )
+
+            if isinstance(memory, int) and memory < 0:
+                raise InvalidUserArgumentException(
+                    "If `memory` value is set as an integer, it must be positive."
+                )
+
+            # We'll need to convert the string value into an integer (in MBs).
+            if isinstance(memory, str):
+                memory = _convert_memory_string_to_mbs(memory)
+
+            assert isinstance(memory, int)
+
+        if gpu_resource_name is not None and (not isinstance(gpu_resource_name, str)):
+            raise InvalidUserArgumentException("`gpu_resource_name` value must be set to a string.")
+
+        spec.resources = ResourceConfig(
+            num_cpus=num_cpus, memory_mb=memory, gpu_resource_name=gpu_resource_name
+        )
+
+
 def op(
     name: Optional[Union[str, UserFunction]] = None,
     description: Optional[str] = None,
@@ -449,46 +527,6 @@ def op(
         description, file_dependencies, requirements, engine, num_outputs, outputs
     )
 
-    resource_config = None
-    if resources is not None:
-        if not isinstance(resources, Dict) or any(not isinstance(k, str) for k in resources):
-            raise InvalidUserArgumentException("`resources` must be a dictionary with string keys.")
-
-        num_cpus = resources.get(NUM_CPUS_KEY)
-        memory = resources.get(MEMORY_KEY)
-        gpu_resource_name = resources.get(GPU_RESOURCE_NAME_KEY)
-
-        if num_cpus is not None and (not isinstance(num_cpus, int) or num_cpus < 0):
-            raise InvalidUserArgumentException(
-                "`num_cpus` value must be set to a positive integer."
-            )
-
-        # `memory` value can be either an int (in MBs) or a string. We will convert it into an integer
-        # representing the number of MBs.
-        if memory is not None:
-            if not isinstance(memory, int) and not isinstance(memory, str):
-                raise InvalidUserArgumentException(
-                    "`memory` value must be either an integer or string."
-                )
-
-            if isinstance(memory, int) and memory < 0:
-                raise InvalidUserArgumentException(
-                    "If `memory` value is set as an integer, it must be positive."
-                )
-
-            # We'll need to convert the string value into an integer (in MBs).
-            if isinstance(memory, str):
-                memory = _convert_memory_string_to_mbs(memory)
-
-            assert isinstance(memory, int)
-
-        if gpu_resource_name is not None and (not isinstance(gpu_resource_name, str)):
-            raise InvalidUserArgumentException("`gpu_resource_name` value must be set to a string.")
-
-        resource_config = ResourceConfig(
-            num_cpus=num_cpus, memory_mb=memory, gpu_resource_name=gpu_resource_name
-        )
-
     def inner_decorator(func: UserFunction) -> OutputArtifactsFunction:
         nonlocal name
         nonlocal description
@@ -528,19 +566,10 @@ def op(
 
             op_spec = OperatorSpec(
                 function=function_spec,
-                resources=resource_config,
             )
 
-            if engine is not None:
-                if globals.__GLOBAL_API_CLIENT__ is None:
-                    raise InvalidUserActionException(
-                        "Aqueduct Client was not instantiated! Please create a client and retry."
-                    )
-
-                op_spec.engine_config = generate_engine_config(
-                    globals.__GLOBAL_API_CLIENT__.list_integrations(),
-                    engine,
-                )
+            _update_operator_spec_with_engine(op_spec, engine)
+            _update_operator_spec_with_resources(op_spec, resources)
 
             assert isinstance(num_outputs, int)
             return wrap_spec(
@@ -586,6 +615,8 @@ def metric(
     file_dependencies: Optional[List[str]] = None,
     requirements: Optional[Union[str, List[str]]] = None,
     output: Optional[str] = None,
+    engine: Optional[str] = None,
+    resources: Optional[Dict[str, Any]] = None,
 ) -> Union[DecoratedMetricFunction, OutputArtifactFunction]:
     """Decorator that converts regular python functions into a metric.
 
@@ -616,6 +647,28 @@ def metric(
         output:
             An optional custom name for the output metric artifact. Otherwise, the default naming scheme
             will be used.
+        engine:
+            The name of the compute integration this operator will run on.
+        resources:
+            A dictionary containing the custom resource configurations that this operator will run with.
+            These configurations are guaranteed to be followed, we will not silently ignore any of them.
+            If a resource configuration is unsupported by a particular execution engine, we will fail at
+            execution time. The supported keys are:
+
+            "num_cpus" (int):
+                The number of cpus that this operator will run with. This operator will execute with *exactly*
+                this number of cpus. If not enough cpus are available, operator execution will fail.
+            "memory" (int, str):
+                The amount of memory this operator will run with. This operator will execute with *exactly*
+                this amount of memory. If not enough memory is available, operator execution will fail.
+
+                If an integer value is supplied, the memory unit is assumed to be MB. If a string is supplied,
+                a suffix indicating the memory unit must be supplied. Supported memory units are "MB" and "GB",
+                case-insensitive.
+
+                For example, the following values are valid: 100, "100MB", "1GB", "100mb", "1gb".
+            "gpu_resource_name" (str):
+                Name of the gpu resource to use (only applicable for Kubernetes engine).
 
     Examples:
         The metric name is inferred from the function name. The description is pulled from the function
@@ -681,10 +734,13 @@ def metric(
             )
 
             metric_spec = MetricSpec(function=function_spec)
+            op_spec = OperatorSpec(metric=metric_spec)
+            _update_operator_spec_with_engine(op_spec, engine)
+            _update_operator_spec_with_resources(op_spec, resources)
 
             output_names = [output] if output is not None else None
             numeric_artifact = wrap_spec(
-                OperatorSpec(metric=metric_spec),
+                op_spec,
                 *artifacts,
                 op_name=name,
                 output_artifact_names=output_names,
@@ -739,6 +795,8 @@ def check(
     file_dependencies: Optional[List[str]] = None,
     requirements: Optional[Union[str, List[str]]] = None,
     output: Optional[str] = None,
+    engine: Optional[str] = None,
+    resources: Optional[Dict[str, Any]] = None,
 ) -> Union[DecoratedCheckFunction, OutputArtifactFunction]:
     """Decorator that converts a regular python function into a check.
 
@@ -772,6 +830,28 @@ def check(
         output:
             An optional custom name for the output metric artifact. Otherwise, the default naming scheme
             will be used.
+        engine:
+            The name of the compute integration this operator will run on.
+        resources:
+            A dictionary containing the custom resource configurations that this operator will run with.
+            These configurations are guaranteed to be followed, we will not silently ignore any of them.
+            If a resource configuration is unsupported by a particular execution engine, we will fail at
+            execution time. The supported keys are:
+
+            "num_cpus" (int):
+                The number of cpus that this operator will run with. This operator will execute with *exactly*
+                this number of cpus. If not enough cpus are available, operator execution will fail.
+            "memory" (int, str):
+                The amount of memory this operator will run with. This operator will execute with *exactly*
+                this amount of memory. If not enough memory is available, operator execution will fail.
+
+                If an integer value is supplied, the memory unit is assumed to be MB. If a string is supplied,
+                a suffix indicating the memory unit must be supplied. Supported memory units are "MB" and "GB",
+                case-insensitive.
+
+                For example, the following values are valid: 100, "100MB", "1GB", "100mb", "1gb".
+            "gpu_resource_name" (str):
+                Name of the gpu resource to use (only applicable for Kubernetes engine).
 
     Examples:
         The check name is inferred from the function name. The description is pulled from the function
@@ -836,10 +916,13 @@ def check(
                 file=zip_file,
             )
             check_spec = CheckSpec(level=severity, function=function_spec)
+            op_spec = OperatorSpec(check=check_spec)
+            _update_operator_spec_with_engine(op_spec, engine)
+            _update_operator_spec_with_resources(op_spec, resources)
 
             output_names = [output] if output is not None else None
             bool_artifact = wrap_spec(
-                OperatorSpec(check=check_spec),
+                op_spec,
                 *artifacts,
                 op_name=name,
                 output_artifact_names=output_names,
