@@ -89,6 +89,12 @@ class S3Config(BaseConnectionConfig):
     use_as_storage: str = "false"
 
 
+class GCSConfig(BaseConnectionConfig):
+    bucket: str
+    service_account_credentials: str
+    use_as_storage: str = "false"
+
+
 class AthenaConfig(BaseConnectionConfig):
     # default type to ACCESS_KEY mainly for backward compatibility
     type: AWSCredentialType = AWSCredentialType.ACCESS_KEY
@@ -114,6 +120,7 @@ class SnowflakeConfig(BaseConnectionConfig):
     database: str
     warehouse: str
     db_schema: Optional[str] = Field("public", alias="schema")  # schema is a Pydantic keyword
+    role: Optional[str] = None
 
     class Config:
         # Ensures that Pydantic parses JSON keys named "schema" or "db_schema" to
@@ -140,6 +147,11 @@ class SlackConfig(BaseConnectionConfig):
     enabled: bool
 
 
+class AWSConfig(BaseConnectionConfig):
+    access_key_id: str
+    secret_access_key: str
+
+
 class _SlackConfigWithStringField(BaseConnectionConfig):
     token: str
     channels_serialized: str
@@ -167,6 +179,10 @@ class _EmailConfigWithStringField(BaseConnectionConfig):
     enabled: str
 
 
+class SparkConfig(BaseConnectionConfig):
+    livy_server_url: str
+
+
 IntegrationConfig = Union[
     BigQueryConfig,
     EmailConfig,
@@ -180,7 +196,9 @@ IntegrationConfig = Union[
     SqlServerConfig,
     SQLiteConfig,
     SlackConfig,
+    AWSConfig,
     _SlackConfigWithStringField,
+    SparkConfig,
 ]
 
 
@@ -211,6 +229,10 @@ def convert_dict_to_integration_connect_config(
         return SlackConfig(**config_dict)
     elif service == ServiceType.EMAIL:
         return EmailConfig(**config_dict)
+    elif service == ServiceType.SPARK:
+        return SparkConfig(**config_dict)
+    elif service == ServiceType.AWS:
+        return AWSConfig(**config_dict)
     raise InternalAqueductError("Unexpected Service Type: %s" % service)
 
 
